import {
  ClassType,
} from '@overflow/commons/core/type';

import {
  InstanceNameType,
  InstanceFactoryType,
} from '@overflow/commons/di/type';

import {InstanceDefinitionRegistry} from './instance_definition_registry';
import {InstanceDefinition} from './instance_definition';

export class InstanceFactory implements InstanceDefinitionRegistry {
  private singletonObjects: Map<InstanceNameType, any> = new Map();
  private singletonFactories: Map<InstanceNameType, InstanceFactoryType> = new Map();
  private instanceDefinitionMap: Map<InstanceNameType, InstanceDefinition> = new Map();
  private instanceDefinitionNames: Set<InstanceNameType> = new Set();

  /**
   * getInstance
   */
  public getInstance(name: InstanceNameType, requiredType: ClassType, ...args: any[]): any {
    if (null === name) {
      return this.getInstanceByType(requiredType);
    }

    let sharedInstance = this.getSingleton(name);
    if (undefined !== sharedInstance) {
      return sharedInstance;
    }


    return null;
  }

  private getInstanceByName(requiredType: ClassType): any {

  }

  private getInstanceByType(requiredType: ClassType): any {

  }

  private getSingleton(name: InstanceNameType): any | undefined {
    let singletonObject = this.singletonObjects.get(name);
  }

  /**
   * containsInstance
   */
  public containsInstance(name: InstanceNameType): boolean {
    return false;
  }

  public registerJSON(json: any): void {
    //
  }

  public registerSingleton(name: InstanceNameType, instanceType: any, instance: any): void {
    //
  }

  public registerInstanceDefinition(name: InstanceNameType, instanceDefinition: InstanceDefinition): void {
    let oldInstanceDefinition = this.instanceDefinitionMap.get(name);
    if (undefined !== oldInstanceDefinition) {
      throw new Error(`InstanceDefinition[${name}] is already exist.`);
    }
    this.instanceDefinitionNames.add(name);
    this.instanceDefinitionMap.set(name, instanceDefinition);
  }

  public removeInstanceDefinition(name: InstanceNameType): void {
    if (!this.instanceDefinitionMap.has(name)) {
      throw new Error(`InstanceDefinition[${name}] is not exist.`);
    }
    this.instanceDefinitionNames.delete(name);
    this.instanceDefinitionMap.delete(name);
  }

  public getInstanceDefinition(name: InstanceNameType): InstanceDefinition | undefined {
    return this.instanceDefinitionMap.get(name);
  }

  public containsInstanceDefinition(name: InstanceNameType): boolean {
    return this.instanceDefinitionMap.has(name);
  }

  public getInstanceDefinitionNames(): Set<InstanceNameType> {
    return this.instanceDefinitionNames;
  }

  public getInstanceDefinitionCount(): number {
    return this.instanceDefinitionMap.size;
  }

  public isInstanceNameInUse(name: InstanceNameType): boolean {
    return false;
  }

}
